#include<math.h>
static const double INFINITE_VALUE = 1.0e15;
static const double INFINITE_DERIV = 1.0e8;
static const double HALF_PI        = 1.570796326794896558;
static const double LOG10          = 2.302585092994045901;
#define         EGE_abs(A)         fabs(A)
#define         EGE_tanh(A)        tanh(A)
#define         EGE_asinh(A)       asinh(A)
#define         EGE_acosh(A)       acosh(A)
#define         EGE_atanh(A)       atanh(A)
#define         EGE_sin(A)         sin(A)
#define         EGE_cos(A)         cos(A)
#define         EGE_tan(A)         tan(A)
#define         EGE_acos(A)        acos(A)
#define         EGE_asin(A)        asin(A)
#define         EGE_atan(A)        atan(A)
#define         EGE_sign(A)        (A >= 0.0 ? 1.0 : -1.0)
#define         EGE_valif(A,B,C)   (A > 0.5 ? B : C)
#define         EGE_sqrt(A)        sqrt(A)
#define         EGE_log(A)         log(A)
#define         EGE_log10(A)       log(A)/LOG10
#define         EGE_sacos_dt(A,B)  EGE_acos_dt(A,B)
#define         EGE_sasin_dt(A,B)  EGE_asin_dt(A,B)
#define         EGE_sacosh_dt(A,B) EGE_acosh_dt(A,B)
#define         EGE_satanh_dt(A,B) EGE_atanh_dt(A,B)
#define         EGE_not(A)         (A == 0.0 ? 1.0 : 0.0)
extern  double  EGE_cosh           (double, char);
extern  double  EGE_cosh_dt        (double, double *, char);
extern  double  EGE_sinh           (double, char);
extern  double  EGE_sinh_dt        (double, double *, char);
extern  double  EGE_tanh_dt        (double, double *, char);
extern  double  EGE_acosh_dt       (double, double *);
extern  double  EGE_atanh_dt       (double, double *);
extern  double  EGE_acos_dt        (double, double *);
extern  double  EGE_asin_dt        (double, double *);
extern  double  EGE_exp            (double, char);
extern  double  EGE_exp_dt         (double, double *, char);
extern  double  EGE_pow            (double, double);
extern  double  EGE_pow_habs       (double, double);
extern  double  EGE_pwr            (double, double);
extern  double  EGE_pwr_compat     (double, double);
extern  double  EGE_fnpow          (double, double);
extern  double  EGE_fnpow_compat   (double, double);
extern  double  EGE_trunc          (double);
extern  double  EGE_round          (double);
extern  double  EGE_smsign_dt      (double, double, double *);
extern  double  EGE_smsign2_dt     (double, double, double, double *, double *);
extern  double  EGE_smmin_dt       (double, double, double, double *, double *);
extern  double  EGE_smmax_dt       (double, double, double, double *, double *);
static __inline double EGE_smabs (double a, double eps) {return(sqrt(a*a + eps));}
static __inline double EGE_div (double a, double b) {return((b == 0.0) ? (a == 0.0 ? 0.0 : INFINITE_VALUE) : a/b);}
static __inline double EGE_div2 (double a, double b) {return((b == 0.0) ? (a == 0.0 ? 0.0 : INFINITE_VALUE) : a/b/b);}
static __inline double EGE_sign2 (double a, double b) {return((b >= 0.0) ? fabs(a) : -fabs(a));}
static __inline double EGE_clog (double a) {return((a > 0.0) ? log(a) : ((a < 0.0) ? -log(-a) : log(1.0e-28)));}
static __inline double EGE_clog10 (double a) {const double ret = ((a > 0.0) ? log(a) : ((a < 0.0) ? -log(-a) : log(1.0e-28)));
  return(ret/LOG10);}
static __inline double EGE_csqrt (double a) {return((a >= 0.0) ? sqrt(a) : -sqrt(-a));}
static __inline double EGE_atan_dt (double a, double * dt) {*dt = 1.0/(1+a*a); return(atan(a));}
static __inline double EGE_slog (double a) {return((a > 0.0) ? log(a) : -INFINITE_VALUE);}
static __inline double EGE_slog_dt (double a, double *dt) {
  if (a > 0.0) {*dt = 1.0/a; return(log(a));}
  *dt = INFINITE_DERIV; return(-INFINITE_VALUE);}
static __inline double EGE_log_dt (double a, double *dt) {*dt = 1.0/a; return(log(a));}
static __inline double EGE_slog10 (double a) {return((a > 0.0) ? log(a)/LOG10 : -INFINITE_VALUE);}
static __inline double EGE_slog10_dt (double a, double *dt) {
  if (a > 0.0) {*dt = 1.0/(a*LOG10); return(log(a)/LOG10);}
  *dt = INFINITE_DERIV; return(-INFINITE_VALUE);}
static __inline double EGE_log10_dt (double a, double *dt) {*dt = 1.0/(a*LOG10); return(log(a)/LOG10);}
static __inline double EGE_clog_dt (double a, double *dt) {
  double ret;
  if (a > 0.0) {ret = log(a); *dt = 1.0/a;}
  else if (a < 0.0) {ret = -log(-a); *dt = 1.0/a;}
  else {ret = log(1.0e-28); *dt = INFINITE_DERIV;}
  return(ret);}
static __inline double EGE_clog10_dt (double a, double *dt) {
  double ret;
  if (a > 0.0) {ret = log(a)/LOG10; *dt = 1.0/(a*LOG10);}
  else if (a < 0.0) {ret = -log(-a)/LOG10; *dt = 1.0/(a*LOG10);}
  else {ret = log(1.0e-28)/LOG10; *dt = INFINITE_DERIV;}
  return(ret);}
static __inline double EGE_csqrt_dt (double a, double *dt) {
  double ret;
  if (a > 0.0) {ret = sqrt(a); *dt = 1.0/(2*ret);}
  else if (a == 0.0) {ret = 0.0; *dt = INFINITE_DERIV;}
  else {ret = -sqrt(-a); *dt = 1.0/(2*ret);}
  return(ret);}
static __inline double EGE_ssqrt (double a) {return((a >= 0.0) ? sqrt(a) : 0.0);}
static __inline double EGE_ssqrt_dt (double a, double *dt) {
  double ret;
  if (a > 0.0) {ret = sqrt(a); *dt = 1.0/(2*ret);}
  else {ret = 0.0; *dt = INFINITE_DERIV;}
  return(ret);}
static __inline double EGE_sqrt_dt (double a, double *dt) {const double ret = sqrt(a); *dt = 1.0/(2*ret); return(ret);}
static __inline double EGE_sgn (double a) {return(a > 0.0 ? 1.0 : (a == 0.0 ? 0.0 : -1.0));}
static __inline double EGE_tan_dt (double a, double *dt) {const double b = cos(a); *dt = 1.0/(b*b); return(tan(a));}
static __inline double EGE_stan (double a) {
  const double b = cos(a);
  return(fabs(b) > 1e-15 ? tan(a) : (EGE_sign(b)*sin(a)/1.0e-15));}
static __inline double EGE_stan_dt (double a, double *dt) {
  const double b = cos(a); double ret;
  if (fabs(b) > 1e-15) {*dt = 1.0/(b*b); ret = tan(a);}
  else {*dt = INFINITE_DERIV; ret = EGE_sign(b)*sin(a)/1.0e-15;}
  return(ret);}
static __inline double EGE_sacos (double a) {return(a >= -1.0 ? (a <= 1.0 ? acos(a) : 0.0) : 2*HALF_PI);}
static __inline double EGE_sasin (double a) {return(a >= -1.0 ? (a <= 1.0 ? asin(a) : HALF_PI) : -HALF_PI);}
static __inline double EGE_sacosh (double a) {return(a > 1.0 ? acosh(a) : 0.0);}
static __inline double EGE_satanh (double a) {return((a > -1.0 && a < 1.0) ? atanh(a) : 0.0);}
static __inline double EGE_min (const double a [], const unsigned int l) {
  double ret = a[0];
  unsigned int i;
  for (i=1; i < l; i++) {
    if (a[i] < ret) {ret = a[i];}
  }
  return(ret);}
static __inline double EGE_min_dt (const double a [], const unsigned int l, unsigned int * imin) {
  double ret = a[0];
  unsigned int i;
  *imin = 0;
  for (i=1; i < l; i++) {
    if (a[i] < ret) {ret = a[i]; *imin = i;}
  }
  return(ret);}
static __inline double EGE_max (const double a [], const unsigned int l) {
  double ret = a[0];
  unsigned int i;
  for (i=1; i < l; i++) {
    if (a[i] > ret) {ret = a[i];}
  }
  return(ret);}
static __inline double EGE_max_dt (const double a [], const unsigned int l, unsigned int * imax) {
  double ret = a[0];
  unsigned int i;
  *imax = 0;
  for (i=1; i < l; i++) {
    if (a[i] > ret) {ret = a[i]; *imax = i;}
  }
  return(ret);}
static __inline double EGE_smmin (double a, double b, double eps) {
  const double d = a - b;
  return(a - 0.5*(d + sqrt(d*d + eps)));}
static __inline double EGE_smmax (double a, double b, double eps) {
   const double d = a - b;
   return(b + 0.5*(d + sqrt(d*d + eps)));}
static __inline double EGE_smsign (double a, double eps) {
  double ret;
  if (a < -eps) {ret = -1.0;}
  else if (a > eps) {ret = 1.0;}
  else {ret = sin(HALF_PI*a/eps);}
  return(ret);}
static __inline double EGE_smsign2 (double a, double b, double eps) {
  double ret;
  if (b < -eps) {ret = -1.0;}
  else if (b > eps) {ret = 1.0;}
  else {ret = sin(HALF_PI*b/eps);}
  return(ret*fabs(a));}
static __inline double EGE_limit (double a, double b, double c) {return(a < b ? b : (a > c ? c : a));}
static __inline double EGE_limit_dt (double a, double b, double c, unsigned int * ider) {
  double ret;
  if (a < b) {ret = b; *ider = 1;}
  else if (a > c) {ret = c; *ider = 2;}
  else {ret = a; *ider = 0;}
  return(ret);}
