----------------------------------------------------------------------
-- Fclkgen.vhd
-- Description:   
-- Author:        Carsten Wulff <wulff@iet.ntnu.no>
-- Created at:    Thu Sep 14 09:18:26 2006
-- Modified at:   Thu Jan 31 11:17:14 2008
-- Modified by:   Carsten Wulff <wulff@iet.ntnu.no>
----------------------------------------------------------------------
-- Clock Generator
-- The clock generator generates 6 clock phases, phi1 and iphi1 are 
-- non-overlapping clocks. Non overlap is usually required in
-- switched capacitor circuits to avoid loosing charge.
-- phi1 (iphi1) is used for sampling switches, those connected to the input
-- phi1a (iphi1a) is used for bottom plate sampling
-- phi2 (iphi2) is used for the multiply switches
-- The variable clk_per defines the clock period of the clock

library IEEE;
use IEEE.math_real.all;
use IEEE.electrical_systems.all;
USE IEEE.STD_LOGIC_1164.ALL;
entity ClkGen is
  generic(
-- Rise and fall times of clock
    rf :time := 100 ps;
    t1 :time := 100 ps;
    period :time := 76923 ps
    );
  port (
    clk               : in  std_logic;
    phi1, phi1a, phi2 : out std_logic;
    iphi1,iphi1a,iphi2 : out std_logic);
  
end ClkGen;


architecture ideal of ClkGen is
  signal clock : std_logic;
  signal clockb : std_logic;
  signal iclock : std_logic;
  signal iclockb : std_logic;
  signal sphi1,sphi1a,sphi2 : std_logic;
  signal isphi1,isphi1a,isphi2 : std_logic;
begin  
  clockb <= Clk;
  iclockb <= not Clk;

  sphi1 <= not clock after rf;
  sphi2 <= not sphi1 after rf;
  sphi1a <= not clockb after rf;

  phi1 <= sphi1;
  phi2 <= sphi2;
  phi1a <= sphi1a;

  isphi1 <= not iclock after rf;
  isphi2 <= not isphi1 after rf;
  isphi1a <= not iclockb after rf;

  iphi1 <= isphi1;
  iphi2 <= isphi2;
  iphi1a <= isphi1a;
  
p1: process
begin
  while true loop
        wait on Clk;
    if( Clk = '1') then
      clock <= '1' after t1, '0' after period/2;
    else
      iclock <= '1' after t1, '0' after period/2;
    end if;
  end loop;
  end process p1;
end ideal;
