----------------------------------------------------------------------
-- Fmdac.vhd
-- Description:   
-- Author:        Carsten Wulff <wulff@iet.ntnu.no>
-- Created at:    Fri Sep 15 08:47:17 2006
-- Modified at:   Thu Jan 31 11:20:25 2008
-- Modified by:   Carsten Wulff <wulff@iet.ntnu.no>
----------------------------------------------------------------------
-- Mulitplying digital to analog converter
-- This is a amplifier that performs the function
-- Vout = Vin*2 - Vref, Vin >= 1/4Vref
-- Vout = Vin*2 , Vin < 1/4Vref & Vin >= -1/4Vref
-- Vout = Vin*2 + Vref, Vin > -1/4Vref
-- It usually entails a switched capacitor circuit 
-- Take a look at http://ieeexplore.ieee.org/search/srchabstract.jsp?arnumber=1458994&isnumber=31404&punumber=4&k2dockey=1458994@ieeejrns
-- for some nice schematics.
library IEEE;
use IEEE.math_real.all;
use IEEE.electrical_systems.all;
USE IEEE.STD_LOGIC_1164.ALL;
library MGC_AMS;
use MGC_AMS.eldo.all;
entity DIG is
  port (
   terminal vcomp, ctrlin : Electrical;     --Analog Input/Output
   signal uclk,phi1,phi1a : in std_logic:='0';     -- Clock Signals
   signal bcurr :inout std_logic_vector (0 to 11) :="000000000000";  --current digital
   signal bout :out std_logic_vector (0 to 11) :="000000000000"  --finished digital out
   );
end DIG;

architecture ideal of DIG is
  type state_type is (SAMP,COMP,B01,B02,B03,B04,B05,B06,B07,B08,B09,B10,B11);
  signal state :state_type:=SAMP;
  signal nextstate :state_type:=SAMP;
  signal b : std_logic_vector (0 to 11) :="000000000000";
  
--  terminal op_ctrlin : electrical;
  -- Define voltages and currents with quantities
  quantity q_vcomp across vcomp;

  quantity q_ctrlin across i_ctrlin through ctrlin;
  -- Internal analog signals
   signal q_op_vcomp : real := 0.0;
   signal s_ctrlin : real := 0.0;
begin

   q_ctrlin  == s_ctrlin'ramp(10.0e-12);
   
   arbiter: process(uclk,phi1)
   variable q_vdd : real :=1.2;
   variable q_vss : real :=0.0;
   variable q_vcm : real :=0.6;
   variable setctrlin : integer := 0;

   begin
   
   if phi1'event and phi1='0' and setctrlin=1 then
	s_ctrlin <= q_vss;
	setctrlin := 0;
	bcurr <= "000000000000";
	end if;
   
   q_op_vcomp <= q_vcomp;
   if uclk'event and uclk='0' then
      --Sampling
      
   CASE state IS
   when SAMP =>
      if(q_op_vcomp > q_vcm) then -- Sett LSB
         --bcurr(11) <= not bcurr(0); --negative input, Va-Vb=positiv
	 b(11) <= not bcurr(0);
      else
         --bcurr(11) <= bcurr(0); --postive input, Va-Vb=negative
	 b(11) <= bcurr(0);
      end if;
      bcurr <= "111111111111";
      s_ctrlin <= q_vdd;
	  nextstate <= COMP;
      
   when COMP =>
	  setctrlin := 1;
      --s_ctrlin <= q_vss;
      --bcurr <= "000000000000";
      nextstate <= B01;
      bout <= b;
      
   when B01 =>
      if(q_op_vcomp > q_vcm) then
         bcurr(0) <= '0'; --negative input, Va-Vb=positive
      else
         bcurr(0) <= '1'; --postive input, Va-Vb=negative
      end if;
      bcurr(1) <= '1'; --trekker fra vrefrange/4
      nextstate <= B02;
      
   
   when B02 =>
      if(q_op_vcomp > q_vcm) then
         bcurr(1) <= not bcurr(0); --Va-Vb=positive
      else
         bcurr(1) <= bcurr(0); --Va-Vb=negative
      end if;
      bcurr(2) <= '1'; --trekker fra vrefrange/4
      nextstate <= B03; 
          
      
   when B03 =>
      if(q_op_vcomp > q_vcm) then
         bcurr(2) <= not bcurr(0); --negative input, Va-Vb=positive
      else
         bcurr(2) <= bcurr(0); --postive input, Va-Vb=negative
      end if;
      bcurr(3) <= '1'; --trekker fra vrefrange/4
        nextstate <= B04;
	
	
   when B04 =>
      if(q_op_vcomp > q_vcm) then
         bcurr(3) <= not bcurr(0); --negative input, Va-Vb=positive
      else
         bcurr(3) <= bcurr(0); --postive input, Va-Vb=negative
      end if;
      bcurr(4) <= '1'; --trekker fra vrefrange/4
           nextstate <= B05;
	   
	   
   when B05 =>
      if(q_op_vcomp > q_vcm) then
         bcurr(4) <= not bcurr(0); --negative input, Va-Vb=positive
      else
         bcurr(4) <= bcurr(0); --postive input, Va-Vb=negative
      end if;
      bcurr(5) <= '1'; --trekker fra vrefrange/4
      nextstate <= B06;
      
      
   when B06 =>
      if(q_op_vcomp > q_vcm) then
         bcurr(5) <= not bcurr(0); --negative input, Va-Vb=positive
      else
         bcurr(5) <= bcurr(0); --postive input, Va-Vb=negative
      end if;
      bcurr(6) <= '1'; --trekker fra vrefrange/4
        nextstate <= B07;
	
	
   when B07 =>
      if(q_op_vcomp > q_vcm) then
         bcurr(6) <= not bcurr(0); --negative input, Va-Vb=positive
      else
         bcurr(6) <= bcurr(0); --postive input, Va-Vb=negative
      end if;
      bcurr(7) <= '1'; --trekker fra vrefrange/4
           nextstate <= B08;
	   
	   
   when B08 =>
      if(q_op_vcomp > q_vcm) then
         bcurr(7) <= not bcurr(0); --negative input, Va-Vb=positive
      else
         bcurr(7) <= bcurr(0); --postive input, Va-Vb=negative
      end if;
      bcurr(8) <= '1'; --trekker fra vrefrange/4
      nextstate <= B09;
      
      
   when B09 =>
      if(q_op_vcomp > q_vcm) then
         bcurr(8) <= not bcurr(0); --negative input, Va-Vb=positive
      else
         bcurr(8) <= bcurr(0); --postive input, Va-Vb=negative
      end if;
      bcurr(9) <= '1'; --trekker fra vrefrange/4
        nextstate <= B10;
        
	
   when B10 =>
      if(q_op_vcomp > q_vcm) then
         bcurr(9) <= not bcurr(0); --negative input, Va-Vb=positive
      else
         bcurr(9) <= bcurr(0); --postive input, Va-Vb=negative
      end if;
      bcurr(10) <= '1'; --trekker fra vrefrange/4
      nextstate <= B11;
      
      
   when B11 =>
      if(q_op_vcomp > q_vcm) then
         bcurr(10) <= not bcurr(0); --negative input, Va-Vb=positive
	 b(10) <= not bcurr(0);
      else
         bcurr(10) <= bcurr(0); --postive input, Va-Vb=negative
	 b(10) <= bcurr(0);
      end if;
      bcurr(11) <= '1'; --trekker fra vrefrange/4           
      nextstate <= SAMP;
      b(0) <= bcurr(0);
      b(1) <= bcurr(1);
      b(2) <= bcurr(2);
      b(3) <= bcurr(3);
      b(4) <= bcurr(4);
      b(5) <= bcurr(5);
      b(6) <= bcurr(6);
      b(7) <= bcurr(7);
      b(8) <= bcurr(8);
      b(9) <= bcurr(9);
      
   end case;
   end if;
   STATE <= NEXTSTATE;
   end process;
end ideal;
