----------------------------------------------------------------------
-- Fdigerrcor.vhd
-- Description:   
-- Author:        Carsten Wulff <wulff@iet.ntnu.no>
-- Created at:    Fri Sep 15 13:23:52 2006
-- Modified at:   Thu Jan 31 11:19:12 2008
-- Modified by:   Carsten Wulff <wulff@iet.ntnu.no>
----------------------------------------------------------------------
----Digital error correction
-- It may sound fancy, but all it does is delay the bits from each stage so
-- they are syncronized, then it adds all the bits in the following fasion:
-- stage 1: 01
-- stage 2:  10 
-- stage 3:   00
-- stage 4:    10
-- stage 5:     01
-- stage 6:      10
-- stage 7:       10
-- stage 8:        01
-- flash:           00
-- Output:  1001101010
-- It should be mentioned that the code in DigErrCorr does this in an highly
-- inefficient manner, but it's behavioral code.
library IEEE;
use IEEE.math_real.all;
use IEEE.electrical_systems.all;
use IEEE.STD_LOGIC_1164.all;
use IEEE.STD_LOGIC_UNSIGNED.all;
entity DigErrCorr is
  port (
    masterclk : in std_logic;
    clk    : in  std_logic;
    iclk   : in  std_logic;
    bb1     : in  std_logic_vector(0 to 1);
    bb2     : in  std_logic_vector(0 to 1);
    bb3     : in  std_logic_vector(0 to 1);
    bb4     : in  std_logic_vector(0 to 1);
    bb5     : in  std_logic_vector(0 to 1);
    bb6     : in  std_logic_vector(0 to 1);
    bb7     : in  std_logic_vector(0 to 1);
    bb8     : in  std_logic_vector(0 to 1) := "00";
    bb9     : in  std_logic_vector(0 to 1) := "00";
    b : out std_logic_vector(0 to 9));
end DigErrCorr;

architecture adms of DigErrCorr is
component shift
       port(C, SI: in  std_logic;
        T: out std_logic_vector(9 downto 0));
    end component;
signal T0,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,T17 : std_logic_vector(9 downto 0) := "0000000000";
signal bs1,bs2,bs3,bs4,bs5,bs6,bs7,bs8,bs9 : std_logic_vector(0 to 9) := "0000000000";
begin  -- adms

  
  dshift0: shift port map (clk ,bb1(0),T0);
  dshift1: shift port map (clk, bb1(1),T1);
  
  dshift2: shift port map (iclk,bb2(0),T2);
  dshift3: shift port map (iclk,bb2(1),T3);

  dshift4: shift port map (clk ,bb3(0),T4);
  dshift5: shift port map (clk ,bb3(1),T5);

  dshift6: shift port map (iclk,bb4(0),T6);
  dshift7: shift port map (iclk,bb4(1),T7);

  dshift8: shift port map (clk ,bb5(0),T8);
  dshift9: shift port map (clk ,bb5(1),T9);

  dshift10: shift port map (iclk,bb6(0),T10);
  dshift11: shift port map (iclk,bb6(1),T11);

  dshift12: shift port map (clk ,bb7(0),T12);
  dshift13: shift port map (clk ,bb7(1),T13);

  dshift14: shift port map (iclk,bb8(0),T14);
  dshift15: shift port map (iclk,bb8(1),T15);

  dshift16: shift port map (clk ,bb9(0),T16);
  dshift17: shift port map (clk ,bb9(1),T17);

  resample: process (masterclk)
  begin       
if masterclk'event and masterclk = '0' then 
       bs1(0) <= T0(8);
       bs1(1) <= T1(8);
       
       bs2(1) <= T2(7);
       bs2(2) <= T3(7);
       
       bs3(2) <= T4(6);
       bs3(3) <= T5(6);
       
       bs4(3) <= T6(5);
       bs4(4) <= T7(5);
       
       bs5(4) <= T8(4);
       bs5(5) <= T9(4);
       
       bs6(5) <= T10(3);
       bs6(6) <= T11(3);
       
       bs7(6) <= T12(2);
       bs7(7) <= T13(2);

       bs8(7) <= T14(1);
       bs8(8) <= T15(1);  

       bs9(8) <= T16(0);
       bs9(9) <= T17(0);
    end if;
  end process resample;

b <= bs1 + bs2 + bs3 + bs4 + bs5 + bs6 + bs7 + bs8 + bs9;
end adms;
