----------------------------------------------------------------------
-- Fflash.vhd
-- Description:   
-- Author:        Carsten Wulff <wulff@iet.ntnu.no>
-- Created at:    Mon Jan  7 16:17:02 2008
-- Modified at:   Thu Jan 31 11:18:39 2008
-- Modified by:   Carsten Wulff <wulff@iet.ntnu.no>
----------------------------------------------------------------------
-- Ideal Flash
-- Since there is no point in doing error correction in the last
-- stage we just use a 2 bit flash, If you want you can use a 1.5 bit comparator
-- instead, which is equal to the SADC, and it won't affect the performance much.
library IEEE;
use IEEE.math_real.all;
use IEEE.electrical_systems.all; 
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.STD_LOGIC_ARITH.ALL;
entity Flash is
  generic (    bits :real := 2.0);
  port (
    terminal vin,vip:  Electrical;
    terminal vcm,vrn,vrp: electrical;
    signal clk : in std_logic;
    signal bb : out std_ulogic_vector(0 to 1));
end Flash;

architecture ideal of Flash is
  quantity q_vin across vin;
  quantity q_vip across vip;
  quantity q_vcm across vcm;
  quantity q_vrn across vrn;
  quantity q_vrp across vrp;
begin 
  
  doadc: process (clk)
    variable v_vin,v_vip,v_in,vref,lsb,val : real := 0.0;
    variable ival : integer := 0;
  begin 
    if clk'event and clk = '0' then
      vref := (q_vrp - q_vrn);
      v_vip := q_vip;
      v_vin := q_vin;
      v_in := (q_vip - q_vin);

      if v_in >= vref/2.0 then
        bb <= "11";
      elsif v_in >= 0.0 then
        bb <= "10";
      elsif v_in >= -vref/2.0 then
        bb <= "01";     
      else
        bb <= "00";
      end if;
    end if;
  end process doadc;
end ideal;
